/*
 *  port.h - defines for generic netlink port handler
 *  Copyright (C) 2015 Cumulus Networks
 */

#ifndef _LINUX_PORT_H
#define _LINUX_PORT_H

#ifdef __KERNEL__
#include <linux/compat.h>
#include <linux/if_bridge.h>
#endif
#include <linux/ethtool.h>
#include <linux/types.h>
#include <net/vxlan.h>

enum {
	PORT_ATTR_UNSPEC,
	PORT_ATTR_STATUS,
	PORT_ATTR_IFINDEX,
	PORT_ATTR_FLAGS,
	PORT_ATTR_CARRIER,
	PORT_ATTR_PHYS_ID_STATE,
	PORT_ATTR_SETTINGS,
	PORT_ATTR_PAUSE,
	PORT_ATTR_MODINFO,
	PORT_ATTR_EEPROM,
	PORT_ATTR_EEPROM_DATA,
	PORT_ATTR_STATS,
	PORT_ATTR_STAT,
	PORT_ATTR_STRINGS,
	PORT_ATTR_STRING,
	PORT_ATTR_SSET,
	PORT_ATTR_SSET_COUNT,
	MC_ATTR_FAMILY,
	MC_ATTR_ARRAY,
	PORT_ATTR_FEC,
	PORT_VSTATS_ATTR_ARRAY,
	PORT_VSTATS_ATTR_OP,
	MC_ATTR_VRF_ARRAY,
	PORT_ATTR_TUNNEL_STATS_ENTRIES,
	PORT_ATTR_TUNNEL_STATS_ENTRY,
	PORT_ATTR_TUNNEL_ID,
	PORT_ATTR_TUNNEL_STATS_RX_BYTES,
	PORT_ATTR_TUNNEL_STATS_RX_PKTS,
	PORT_ATTR_TUNNEL_STATS_RX_DROPS,
	PORT_ATTR_TUNNEL_STATS_RX_ERRORS,
	PORT_ATTR_TUNNEL_STATS_TX_BYTES,
	PORT_ATTR_TUNNEL_STATS_TX_PKTS,
	PORT_ATTR_TUNNEL_STATS_TX_DROPS,
	PORT_ATTR_TUNNEL_STATS_TX_ERRORS,
	PORT_ATTR_TUNNEL_OPTIONS,
	__PORT_ATTR_MAX,
};

#define PORT_ATTR_MAX (__PORT_ATTR_MAX - 1)

enum {
	PORT_CMD_UNSPEC,
	PORT_CMD_REPLY,
	PORT_CMD_GET_SETTINGS,
	PORT_CMD_SET_SETTINGS,
	PORT_CMD_GET_PAUSE,
	PORT_CMD_SET_PAUSE,
	PORT_CMD_GET_MODULE_INFO,
	PORT_CMD_GET_MODULE_EEPROM,
	PORT_CMD_SET_STATS,
	PORT_CMD_GET_STRINGS,
	PORT_CMD_GET_SSET_COUNT,
	PORT_CMD_SET_CARRIER,
	PORT_CMD_SET_PHYS_ID_STATE,
	PORT_CMD_GET_CACHED_SETTINGS,
	MC_CMD_REFRESH_AGE,
	PORT_CMD_GET_FEC,
	PORT_CMD_SET_FEC,
	PORT_CMD_UPDATE_VSTATS,
	PORT_CMD_GET_CACHED_FEC,
	PORT_CMD_UPDATE_TUNNEL_STATS,
	__PORT_CMD_MAX,
};

union mc_addr {
	struct {
		__be32 group;
		__be32 source;
	};
	struct {
		struct in6_addr v6group;
		struct in6_addr v6source;
	};
};

struct mc_addr_vrf {
	union mc_addr mcaddr;
	u32 table;
	u64 unused;
};

enum port_swp_stats {
	HwIfInOctets,
	HwIfInUcastPkts,
	HwIfInBcastPkts,
	HwIfInMcastPkts,
	HwIfOutOctets,
	HwIfOutUcastPkts,
	HwIfOutMcastPkts,
	HwIfOutBcastPkts,
	HwIfInDiscards,
	HwIfInL3Drops,
	HwIfInBufferDrops,
	HwIfInAclDrops,
	HwIfInDot3LengthErrors,
	HwIfInErrors,
	SoftInErrors,
	SoftInDrops,
	SoftInFrameErrors,
	HwIfOutDiscards,
	HwIfOutErrors,
	HwIfOutQDrops,
	HwIfOutNonQDrops,
	SoftOutErrors,
	SoftOutDrops,
	SoftOutTxFifoFull,
	HwIfOutQLen,
	HwIfInDot3FrameErrors,
	HwIfInPausePkt,
	HwIfOutPausePkt,
	HwIfInPfc0Pkt,
	HwIfOutPfc0Pkt,
	HwIfInPfc1Pkt,
	HwIfOutPfc1Pkt,
	HwIfInPfc2Pkt,
	HwIfOutPfc2Pkt,
	HwIfInPfc3Pkt,
	HwIfOutPfc3Pkt,
	HwIfInPfc4Pkt,
	HwIfOutPfc4Pkt,
	HwIfInPfc5Pkt,
	HwIfOutPfc5Pkt,
	HwIfInPfc6Pkt,
	HwIfOutPfc6Pkt,
	HwIfInPfc7Pkt,
	HwIfOutPfc7Pkt,
	HwIfOutWredDrops,
	HwIfOutQ0WredDrops,
	HwIfOutQ1WredDrops,
	HwIfOutQ2WredDrops,
	HwIfOutQ3WredDrops,
	HwIfOutQ4WredDrops,
	HwIfOutQ5WredDrops,
	HwIfOutQ6WredDrops,
	HwIfOutQ7WredDrops,
	HwIfOutQ8WredDrops,
	HwIfOutQ9WredDrops,
	HwIfAlignmentErrors,
	HwIfSymbolErrors,
	HwIfOversizeErrors,
	HwIfUndersizeErrors,
	HwIfJabberErrors,
	HwIfIn64OctetsPkts,
	HwIfIn65To127OctetsPkts,
	HwIfIn128To255OctetsPkts,
	HwIfIn256To511OctetsPkts,
	HwIfIn512To1023OctetsPkts,
	HwIfIn1024To1518OctetsPkts,
	HwIfIn1519To2047OctetsPkts,
	HwIfIn2048To4095OctetsPkts,
	HwIfIn4096To16383OctetsPkts,
	HwIfOut64OctetsPkts,
	HwIfOut65To127OctetsPkts,
	HwIfOut128To255OctetsPkts,
	HwIfOut256To511OctetsPkts,
	HwIfOut512To1023OctetsPkts,
	HwIfOut1024To1518OctetsPkts,
	HwIfOut1519To2047OctetsPkts,
	HwIfOut2048To4095OctetsPkts,
	HwIfOut4096To16383OctetsPkts,
};

enum port_tunnel_options {
	PORT_TUNNEL_OPT_IGN_UNK_IFINDEX, /* ignore unknown ifindex */
	PORT_TUNNEL_OPT_IGN_UNK_ID, /* ignore unknown ID */
	__PORT_TUNNEL_OPT_MAX,
};

#ifdef __KERNEL__

#define PORT_ATTR_FLAG_STAT_RESTART BIT(0)
#define PORT_ATTR_FLAG_CARRIER_SET_QUEUE BIT(1)

#if defined(CONFIG_SYSCTL) && defined(CONFIG_NET_PORT)
extern int cl_hw_accel;
#else
#define cl_hw_accel 1
#endif

struct port_tunnel_stats {
	struct vxlan_vni_stats vxstats;
	/* synchronize with stats updates */
	struct rcu_head rcu_head;
};

#ifdef CONFIG_NET_PORT
void port_init_ethtool_stats(struct net_device *dev);
void port_uninit_ethtool_stats(struct net_device *dev);
int port_get_link_settings(struct net_device *dev, struct ethtool_link_ksettings *link_ksettings);
int port_set_link_settings(struct net_device *dev, const struct ethtool_link_ksettings *link_ksettings);
void port_get_pauseparam(struct net_device *dev,
			 struct ethtool_pauseparam *pause);
int port_set_pauseparam(struct net_device *dev,
			struct ethtool_pauseparam *pause);
void port_get_ethtool_stats(struct net_device *dev,
			    struct ethtool_stats *stats, u64 *data);
void port_get_ethtool_stats_clear(struct net_device *dev,
				  struct ethtool_stats *stats, u64 *data);
void port_augment_dev_ethtool_stats(struct net_device *dev,
				    const struct rtnl_link_stats64 *sx_stats,
				    struct rtnl_link_stats64 *stats);
void port_get_strings(struct net_device *dev, u32 stringset, u8 *data);
int port_get_sset_count(struct net_device *dev, int sset);
int port_set_phys_id(struct net_device *dev, enum ethtool_phys_id_state state);
int port_get_module_info(struct net_device *dev, struct ethtool_modinfo *info);
int port_get_module_eeprom(struct net_device *dev,
			   struct ethtool_eeprom *eeprom, u8 *data);
int port_get_fecparam(struct net_device *dev,
			 struct ethtool_fecparam *fecparam);
int port_set_fecparam(struct net_device *dev,
			struct ethtool_fecparam *fecparam);
void port_vstats_fetch(struct bridge_vlan_xstats *vstore);
void port_vstats_reset(u16 vid);
bool port_vstats_valid(void);

int port_tunnel_add(struct net_device *dev, u32 tunid);
int port_tunnel_del(struct net_device *dev, u32 tunid);
const struct port_tunnel_stats *port_tunnel_get(struct net_device *dev,
						u32 tunid);
int port_devlink_port_split(struct devlink *devlink,
			    unsigned int port_index,
			    unsigned int count,
			    struct netlink_ext_ack* extack);
int port_devlink_port_unsplit(struct devlink *devlink,
			      unsigned int port_index,
			      struct netlink_ext_ack* extack);
#else /* CONFIG_NET_PORT */
static inline void port_init_ethtool_stats(struct net_device *dev) {}
static inline void port_uninit_ethtool_stats(struct net_device *dev) {}
static inline int port_get_link_settings(struct net_device *dev,
			struct ethtool_link_ksettings *link_ksettings)
{
	return 0;
}
static inline int port_set_link_settings(struct net_device *dev,
		const struct ethtool_link_ksettings *link_ksettings)
{
	return 0;
}
static inline void port_get_pauseparam(struct net_device *dev,
			 struct ethtool_pauseparam *pause) {}
static inline int port_set_pauseparam(struct net_device *dev,
			struct ethtool_pauseparam *pause)
{
	return 0;
}
static inline void port_get_ethtool_stats(struct net_device *dev,
			    struct ethtool_stats *stats, u64 *data) {}
static inline void port_get_ethtool_stats_clear(struct net_device *dev,
				  struct ethtool_stats *stats, u64 *data) {}
static inline void port_augment_dev_ethtool_stats(struct net_device *dev,
				    const struct rtnl_link_stats64 *sx_stats,
				    struct rtnl_link_stats64 *stats) {}
static inline void port_get_strings(struct net_device *dev,
				    u32 stringset,
				    u8 *data) {}
static inline int port_get_sset_count(struct net_device *dev, int sset)
{
	return 0;
}
static inline int port_set_phys_id(struct net_device *dev,
				   enum ethtool_phys_id_state state)
{
	return 0;
}
static inline int port_get_module_info(struct net_device *dev,
				       struct ethtool_modinfo *info)
{
	return 0;
}
static inline int port_get_module_eeprom(struct net_device *dev,
			   struct ethtool_eeprom *eeprom, u8 *data)
{
	return 0;
}
static inline int port_get_fecparam(struct net_device *dev,
			 struct ethtool_fecparam *fecparam)
{
	return 0;
}
static inline int port_set_fecparam(struct net_device *dev,
			struct ethtool_fecparam *fecparam)
{
	return 0;
}
static inline void port_vstats_fetch(struct bridge_vlan_xstats *vstore) {}
static inline void port_vstats_reset(u16 vid) {}
static inline bool port_vstats_valid(void)
{
	return false;
}

static inline int port_tunnel_add(struct net_device *dev, u32 tunid)
{
	return 0;
}
static inline int port_tunnel_del(struct net_device *dev, u32 tunid)
{
	return 0;
}
static inline const struct port_tunnel_stats *port_tunnel_get(
	struct net_device *dev, u32 tunid)
{
	WARN_ON_ONCE(!rcu_read_lock_held());
	return NULL;
}
int port_devlink_port_split(struct devlink *devlink,
			    unsigned int port_index,
			    unsigned int count,
			    struct netlink_ext_ack* extack)
{
	return 0;
}
int port_devlink_port_unsplit(struct devlink *devlink,
			      unsigned int port_index,
			      struct netlink_ext_ack* extack)
{
	return 0;
}
#endif /* CONFIG_NET_PORT */
#endif /* __KERNEL__ */

#endif /* _LINUX_PORT_H */
